import { Utilities } from '@/util/Utilities';
import {
    Form, LoadingService, Modal, ModalService, Password, TextInput,
} from '@zeedhi/common';
import {
    Config, Http, IDictionary, IEventParam, Metadata, Singleton,
} from '@zeedhi/core';

@Singleton
export class LoginController {
    get logo() {
        return Config.env.product.logoUrl;
    }

    get backgroundImgUrl() {
        const url = Config.env.loginBackgroundImgUrl;
        return `url(${url})`;
    }

    get endPoint() {
        return Config.env.loginEndPoint;
    }

    private signUpModal!: Modal;

    public async openSignUpModal() {
        this.signUpModal = await ModalService.createFromJson('/sign_up_modal', true);
        this.signUpModal.show();
    }

    public closeSignUpModal() {
        this.signUpModal.hide();
        this.signUpModal?.destroy();
    }

    public validateEmail({ component }: IEventParam<TextInput>) {
        // Expressão regular para validar um e-mail
        const regex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
        if (!regex.test(component.value)) {
            Utilities.alertMessage('Lembre-se de colocar um email válido @email.com', 'error');
        }
    }

    public validatePassword() {
        const senha = Metadata.getInstance<Password>('password-cadastro').value;
        const senha2 = Metadata.getInstance<Password>('password-cadastro-2').value;

        if ((senha && senha2) && (senha !== senha2)) {
            Utilities.alertMessage('As senhas não coincidem', 'error');
            return false;
        }
        return true;
    }

    public signUp() {
        const form = Metadata.getInstance<Form>('form-cadastro');

        if (this.validatePassword() && Utilities.validateForm(form)) {
            Http.post('/signUp', {
                NAME: Metadata.getInstance<TextInput>('nome-completo').value,
                EMAIL: Metadata.getInstance<TextInput>('email-cadastro').value,
                PASSWORD: Metadata.getInstance<Password>('password-cadastro').value,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.closeSignUpModal();
            });
        }
    }

    public signIn() {
        const form = Metadata.getInstance<Form>('loginFormSocialLogin');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/signIn', {
                EMAIL: Metadata.getInstance<TextInput>('emailSocialLogin').value,
                PASSWORD: Metadata.getInstance<Password>('passwordSocialLogin').value,
            }).then((response: IDictionary) => {
                if (response.data) {
                    Utilities.showSuccessMessage();
                    localStorage.setItem('LOGIN_TOKEN', response.data.LOGIN_TOKEN);
                    localStorage.setItem('LOGIN_USER_NAME', `${response.data.DSPRIMEIRONOME} ${response.data.DSULTIMONOME}`);
                    localStorage.setItem('LOGIN_USER', response.data.LOGIN_USER);
                    window.location.href = '/';
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            })
            .finally(() => {
                LoadingService.hide();
            });
        }
    }

    public getGoogleAuthUrl() {
        Http.get('/getGoogleAuthUrl').then((response: IDictionary) => {
            if (response.data) {
                window.location.href = response.data;
                // window.open(response.data);
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        })
        .finally(() => {
            LoadingService.hide();
        });
    }

    public redirectGoogle() {
        const retorno = window.location.search;
        if (retorno !== '') {
            const params = new URLSearchParams(window.location.search);
            LoadingService.show();
            Http.post('/callback-google', {
                code: params.get('code'),
            }).then((response: IDictionary) => {
                if (response.data) {
                    console.warn(response.data);
                    Utilities.showSuccessMessage();
                    localStorage.setItem('LOGIN_TOKEN', response.data.LOGIN_TOKEN);
                    localStorage.setItem('LOGIN_USER_NAME', `${response.data.DSPRIMEIRONOME} ${response.data.DSULTIMONOME}`);
                    localStorage.setItem('LOGIN_USER', response.data.LOGIN_USER);
                    window.location.href = '/';
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            })
            .finally(() => {
                LoadingService.hide();
            });
        }
    }
}
